import HuffmanTree, { Comparable } from "./HuffmanTree";

export default class HuffmanCoding<T extends Comparable<T>> {
  private _hufftree: HuffmanTree<T> | undefined;
  get hufftree(): HuffmanTree<T> {
    if (!this._hufftree) {
      this._hufftree = new HuffmanTree(this.originalInput || this.input);
    }
    return this._hufftree;
  }
  // public plaintext: string = "";
  private _encodedString: string = "";
  // public originText?: string;
  public get encodedString(): string {
    if (!this._encodedString && this.input.length > 0) {
      this._encodedString = this.encode();
    }
    return this._encodedString;
  }
  public originalInput: T[] = [];
  public input: T[] = [];
  public compressionRatio(inputSize: number) {
    return (this.input.length * inputSize) / this.encodedString.length;
  }
  private constructor() {}
  public static fromTree<K extends Comparable<K>>(tree: HuffmanTree<K>) {
    const a = new HuffmanCoding<K>();
    a._hufftree = tree;
    a.originalInput = tree.message;
    a.input = a.originalInput;
    return a;
  }
  public static fromPlainText(plaintext: string) {
    const a = new HuffmanCoding<string>();
    a.input = plaintext.split("");
    a.originalInput = a.input;
    a._hufftree = new HuffmanTree(a.originalInput);
    return a;
  }
  encode(message: T[] = this.input): string {
    this.input = message;
    const dic = this.hufftree.huffmanCodes;
    const r: string[] = [];
    for (let i = 0; i < message.length; i++) {
      const element = message[i];
      r.push(dic.get(element) as string);
    }
    this._encodedString = r.join("");
    return this.encodedString;
  }
  decode(encoded: string): T[] {
    this._encodedString = encoded;
    const arr = encoded.split("");
    const r: T[] = [];
    const dic = reverseMap(this.hufftree.huffmanCodes);
    for (let i = 0; i < arr.length; i++) {
      let element = arr[i];
      let v = dic.get(element);
      while (!v) {
        element += arr[++i];
        v = dic.get(element);
      }
      r.push(v);
    }
    this.input = r; // = r.join("");
    return this.input;
  }
}
function reverseMap<K, V>(m: Map<K, V>): Map<V, K> {
  const r = new Map<V, K>();
  m.forEach((v, k) => {
    r.set(v, k);
  });
  return r;
}
