"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const HuffmanTree_1 = __importDefault(require("./HuffmanTree"));
class HuffmanCoding {
    constructor() {
        // public plaintext: string = "";
        this._encodedString = "";
        this.originalInput = [];
        this.input = [];
    }
    get hufftree() {
        if (!this._hufftree) {
            this._hufftree = new HuffmanTree_1.default(this.originalInput || this.input);
        }
        return this._hufftree;
    }
    // public originText?: string;
    get encodedString() {
        if (!this._encodedString && this.input.length > 0) {
            this._encodedString = this.encode();
        }
        return this._encodedString;
    }
    compressionRatio(inputSize) {
        return (this.input.length * inputSize) / this.encodedString.length;
    }
    static fromTree(tree) {
        const a = new HuffmanCoding();
        a._hufftree = tree;
        a.originalInput = tree.message;
        a.input = a.originalInput;
        return a;
    }
    static fromPlainText(plaintext) {
        const a = new HuffmanCoding();
        a.input = plaintext.split("");
        a.originalInput = a.input;
        a._hufftree = new HuffmanTree_1.default(a.originalInput);
        return a;
    }
    encode(message = this.input) {
        this.input = message;
        const dic = this.hufftree.huffmanCodes;
        const r = [];
        for (let i = 0; i < message.length; i++) {
            const element = message[i];
            r.push(dic.get(element));
        }
        this._encodedString = r.join("");
        return this.encodedString;
    }
    decode(encoded) {
        this._encodedString = encoded;
        const arr = encoded.split("");
        const r = [];
        const dic = reverseMap(this.hufftree.huffmanCodes);
        for (let i = 0; i < arr.length; i++) {
            let element = arr[i];
            let v = dic.get(element);
            while (!v) {
                element += arr[++i];
                v = dic.get(element);
            }
            r.push(v);
        }
        this.input = r; // = r.join("");
        return this.input;
    }
}
exports.default = HuffmanCoding;
function reverseMap(m) {
    const r = new Map();
    m.forEach((v, k) => {
        r.set(v, k);
    });
    return r;
}
