"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class HuffmanTree {
    constructor(message) {
        this.message = message;
        // this._frequencies = frequencies(message.split(""));
        this._frequencies = frequencies(message);
    }
    get tree() {
        if (!this._tree) {
            this._tree = this.mkTree();
        }
        return this._tree;
    }
    get keyLengths() {
        if (!this._keyLengths) {
            this._keyLengths = this.toKeyLengths();
        }
        return this._keyLengths;
    }
    get huffmanCodes() {
        if (!this._huffmanCodes) {
            this._huffmanCodes = this.toHuffmanCodes();
        }
        return this._huffmanCodes;
    }
    mkTree() {
        const queue = [];
        this._frequencies.forEach((n, str) => {
            queue.push({ count: n, literal: str });
        });
        const lowest = (queue) => {
            const lowest1 = queue.reduce((lowest, curr) => curr.count < lowest.count ? curr : lowest);
            queue.splice(queue.indexOf(lowest1), 1);
            return lowest1;
        };
        while (queue.length > 1) {
            const lowest1 = lowest(queue);
            const lowest2 = lowest(queue);
            queue.push({
                count: lowest1.count + lowest2.count,
                leftTree: lowest1,
                rightTree: lowest2
            });
        }
        // this._tree = queue[0];
        return queue[0];
    }
    toKeyLengths() {
        const tree = this.tree;
        const m = new Map();
        function f(tree, len = 0) {
            if (tree && tree.literal) {
                m.set(tree.literal, len);
            }
            else {
                if (tree) {
                    if (tree.leftTree) {
                        f(tree.leftTree, len + 1);
                    }
                    if (tree.rightTree) {
                        f(tree.rightTree, len + 1);
                    }
                }
            }
        }
        f(tree);
        return m;
    }
    toHuffmanCodes() {
        // const keyLens = arrayify(this.keyLengths);
        const dic = new Map();
        let curlen = 1;
        // let curSet = keyLens.filter(x => x.value == curlen);
        let bl_count = [0];
        this.keyLengths.forEach(v => {
            bl_count[v] = (bl_count[v] || 0) + 1;
        });
        // while (keyLens.length > 0) {
        //   bl_count[curlen] = curSet.length;
        //   curSet.forEach(x => {
        //     keyLens.splice(keyLens.indexOf(x), 1);
        //   });
        //   curlen++;
        //   curSet = keyLens.filter(x => x.value == curlen);
        // }
        console.log("bl_count is", bl_count);
        let code = 0;
        const MAX_BITS = bl_count.length - 1;
        let next_code = [];
        for (let bits = 1; bits <= MAX_BITS; bits++) {
            code = (code + bl_count[bits - 1]) << 1;
            next_code[bits] = code;
        }
        console.log("next_code", next_code.map(dec => (dec >>> 0).toString(2)));
        //ready for codes.
        const ls = arrayify(this.keyLengths);
        ls.sort((a, b) => a.key.localeCompare(b.key));
        //now in alphabetical order!
        ls.forEach(x => {
            const char = x.key;
            const len = x.value;
            dic.set(char, withLength((next_code[len] >>> 0).toString(2), len));
            next_code[len] += 1;
        });
        return dic;
    }
}
exports.default = HuffmanTree;
function withLength(str, len) {
    return "0".repeat(Math.max(0, len - str.length)) + str;
}
function nextCode(codes, length) { }
function arrayify(map) {
    const r = [];
    map.forEach((v, k) => {
        r.push({ key: k, value: v });
    });
    return r;
}
function frequencies(message) {
    // const ls = message.split("");
    const ls = message;
    const dic = new Map();
    ls.forEach((x, i) => {
        const e = dic.get(x) || 0;
        dic.set(x, e + 1);
    });
    return dic;
}
